@@ -2,6 +2,7 @@
 """Docstring."""
 from . import pipelines
 from . import transforms
+from .constants import ADC_GAIN
 from .constants import AMPLITUDE_UUID
 from .constants import AUC_UUID
 from .constants import BESSEL_BANDPASS_UUID
@@ -9,12 +10,19 @@ from .constants import BESSEL_LOWPASS_10_UUID
 from .constants import BESSEL_LOWPASS_30_UUID
 from .constants import BUTTERWORTH_LOWPASS_30_UUID
 from .constants import CENTIMILLISECONDS_PER_SECOND
+from .constants import CONTRACTION_VELOCITY_UUID
 from .constants import MIDSCALE_CODE
+from .constants import MILLI_TO_BASE_CONVERSION
+from .constants import MILLIMETERS_PER_MILLITESLA
+from .constants import MILLIVOLTS_PER_MILLITESLA
 from .constants import MIN_NUMBER_PEAKS
 from .constants import MIN_NUMBER_VALLEYS
+from .constants import NEWTONS_PER_MILLIMETER
 from .constants import PRIOR_PEAK_INDEX_UUID
 from .constants import PRIOR_VALLEY_INDEX_UUID
 from .constants import RAW_TO_SIGNED_CONVERSION_VALUE
+from .constants import REFERENCE_VOLTAGE
+from .constants import RELAXATION_VELOCITY_UUID
 from .constants import SUBSEQUENT_PEAK_INDEX_UUID
 from .constants import SUBSEQUENT_VALLEY_INDEX_UUID
 from .constants import TWITCH_FREQUENCY_UUID
@@ -37,11 +45,13 @@ from .transforms import apply_empty_plate_calibration
 from .transforms import apply_noise_filtering
 from .transforms import apply_sensitivity_calibration
 from .transforms import calculate_displacement_from_voltage
+from .transforms import calculate_force_from_displacement
 from .transforms import calculate_voltage_from_gmr
 from .transforms import create_filter
 from .transforms import FILTER_CHARACTERISTICS
 from .transforms import noise_cancellation
 
+
 if (
     5 < 10
 ):  # pragma: no cover # protect this from zimports deleting the pylint disable statement
@@ -92,4 +102,13 @@ __all__ = [
     "MIN_NUMBER_VALLEYS",
     "BESSEL_LOWPASS_30_UUID",
     "BUTTERWORTH_LOWPASS_30_UUID",
+    "RELAXATION_VELOCITY_UUID",
+    "CONTRACTION_VELOCITY_UUID",
+    "calculate_force_from_displacement",
+    "MILLI_TO_BASE_CONVERSION",
+    "MILLIVOLTS_PER_MILLITESLA",
+    "MILLIMETERS_PER_MILLITESLA",
+    "NEWTONS_PER_MILLIMETER",
+    "REFERENCE_VOLTAGE",
+    "ADC_GAIN",
 ]
