@@ -3,6 +3,7 @@
 import uuid
 
 CENTIMILLISECONDS_PER_SECOND = 100000
+
 TWITCH_PERIOD_UUID = uuid.UUID("6e0cd81c-7861-4c49-ba14-87b2739d65fb")
 TWITCH_FREQUENCY_UUID = uuid.UUID(
     "472d0707-ff87-4198-9374-c28900bb216c"
@@ -13,6 +14,8 @@ WIDTH_UUID = uuid.UUID("c4c60d55-017a-4783-9600-f19606de26f3")
 WIDTH_VALUE_UUID = uuid.UUID("05041f4e-c77d-42d9-a2ae-8902f912e9ac")
 WIDTH_RISING_COORDS_UUID = uuid.UUID("2a16acb6-4df7-4064-9d47-5d27ea7a98ad")
 WIDTH_FALLING_COORDS_UUID = uuid.UUID("26e5637d-42c9-4060-aa5d-52209b349c84")
+RELAXATION_VELOCITY_UUID = uuid.UUID("0fcc0dc3-f9aa-4f1b-91b3-e5b5924279a9")
+CONTRACTION_VELOCITY_UUID = uuid.UUID("73961e7c-17ec-42b0-b503-a23195ec249c")
 
 PRIOR_PEAK_INDEX_UUID = uuid.UUID("80df90dc-21f8-4cad-a164-89436909b30a")
 PRIOR_VALLEY_INDEX_UUID = uuid.UUID("72ba9466-c203-41b6-ac30-337b4a17a124")
@@ -25,12 +28,16 @@ BESSEL_LOWPASS_30_UUID = uuid.UUID("eee66c75-4dc4-4eb4-8d48-6c608bf28d91")
 BUTTERWORTH_LOWPASS_30_UUID = uuid.UUID("de8d8cef-65bf-4119-ada7-bdecbbaa897a")
 
 # GMR conversion factors
+# Conversion values were obtained 03/09/2021 by Kevin Grey
 MIDSCALE_CODE = 0x800000
 RAW_TO_SIGNED_CONVERSION_VALUE = 2 ** 23  # subtract this value from raw hardware data
+MILLIVOLTS_PER_MILLITESLA = 1073.6
+MILLIMETERS_PER_MILLITESLA = 23.25
+NEWTONS_PER_MILLIMETER = 0.000159
+REFERENCE_VOLTAGE = 2.5
+ADC_GAIN = 2
+MILLI_TO_BASE_CONVERSION = 1000
 
-# REFERENCE_VOLTAGE = 3.3
-
-# ADC_GAIN = 32
 
 MIN_NUMBER_PEAKS = 3
 MIN_NUMBER_VALLEYS = 3
